
/* Two-Dimensional Life				*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/

# define LA	   100  /* edge of square of evolution		*/
# define KK	     2  /* number of states per cell		*/
# define RR	     1  /* radius of neighborhood		*/
# define TS	    50  /* number of sums in totalistic rule	*/

char sqrule[102];

/* calculate one generation of evolution */
lifegen(a1, a2) char a1[LA][LA], a2[LA][LA]; {int i0, i1;

/* clear tally array */
for (i0=0; i0<LA; i0++) for (i1=0; i1<LA; i1++) a2[i0][i1]=0;

/* count neighbors of main square */
phrame(1); /* phase to count neighbors */

for (i0=1; i0<LA-1; i0++)
for (i1=1; i1<LA-1; i1++)
  if (a1[i0][i1]!=0) {
        a2[i0-1][i1-1]++;
        a2[i0-1][i1+1]++;
        a2[i0+1][i1-1]++;
        a2[i0+1][i1+1]++;
        a2[i0-1][i1]+=5;
        a2[i0][i1-1]+=5;
        a2[i0][i1+1]+=5;
        a2[i0+1][i1]+=5;
        a2[i0][i1]+=25;
        }

/* count neighbors for left edge */

for (i0=1; i0<LA-1; i0++) {
  if (a1[i0][0]==1) {
    a2[i0-1][LA-1]++;
    a2[i0-1][1]++;
    a2[i0+1][LA-1]++;
    a2[i0+1][1]++;
    a2[i0-1][0]+=5;
    a2[i0][LA-1]+=5;
    a2[i0][1]+=5;
    a2[i0+1][0]+=5;
    a2[i0][0]+=25;
    };
  };

/* count neighbors for right edge */

for (i0=1; i0<LA-1; i0++) {
  if (a1[i0][LA-1]==1) {
    a2[i0-1][LA-2]++;
    a2[i0-1][0]++;
    a2[i0+1][LA-2]++;
    a2[i0+1][0]++;
    a2[i0-1][LA-1]+=5;
    a2[i0][LA-2]+=5;
    a2[i0][0]+=5;
    a2[i0+1][LA-1]+=5;
    a2[i0][LA]+=25;
    };
  };

/* count neighbors for top edge */

for (i1=1; i1<LA-1; i1++) {
  if (a1[0][i1]==1) {
    a2[LA-1][i1-1]++;
    a2[LA-1][i1+1]++;
    a2[1][i1-1]++;
    a2[1][i1+1]++;
    a2[LA-1][i1]+=5;
    a2[0][i1-1]+=5;
    a2[0][i1+1]+=5;
    a2[1][i1]+=5;
    a2[0][i1]+=25;
    };
  };

/* count neighbors of bottom edge */

for (i1=1; i1<LA-1; i1++) {
  if (a1[LA-1][i1]==1) {
    a2[LA-2][i1-1]++;
    a2[LA-2][i1+1]++;
    a2[0][i1-1]++;
    a2[0][i1+1]++;
    a2[LA-2][i1]+=5;
    a2[LA-1][i1-1]+=5;
    a2[LA-1][i1+1]+=5;
    a2[0][i1]+=5;
    a2[LA-1][i1]+=25;
    };
  };

/* count neighbors for upper left hand corner */

  if (a1[0][0]==1) {
    a2[LA-1][LA-1]++;
    a2[LA-1][1]++;
    a2[1][LA-1]++;
    a2[1][1]++;
    a2[LA-1][0]+=5;
    a2[0][LA-1]+=5;
    a2[0][1]+=5;
    a2[1][0]+=5;
    a2[0][0]+=25;
    };

/* count neighbors for upper right hand corner */

  if (a1[0][LA-1]==1) {
    a2[LA-1][LA-2]++;
    a2[LA-1][0]++;
    a2[1][LA-2]++;
    a2[1][0]++;
    a2[LA-1][LA-1]+=5;
    a2[0][LA-2]+=5;
    a2[0][0]+=5;
    a2[1][LA-1]+=5;
    a2[0][LA-1]+=25;
    };

/* count neighbors for bottom left hand corner */

  if (a1[LA-1][0]==1) {
    a2[LA-2][LA-2]++;
    a2[LA-2][1]++;
    a2[0][LA-2]++;
    a2[0][1]++;
    a2[LA-2][0]+=5;
    a2[LA-1][LA-2]+=5;
    a2[LA-1][1]+=5;
    a2[0][0]+=5;
    a2[LA-1][0]+=25;
    };

/* count neighbors for bottom right hand corner */

  if (a1[LA-1][LA-1]==1) {
    a2[LA-2][LA-2]++;
    a2[LA-2][0]++;
    a2[0][LA-2]++;
    a2[0][0]++;
    a2[LA-2][LA-1]+=5;
    a2[LA-1][LA-2]+=5;
    a2[LA-1][0]+=5;
    a2[0][LA-1]+=5;
    a2[LA-1][LA-1]+=25;
    };

phrame(2); /* phase to update cells */
for (i0=0; i0<LA; i0++)
for (i1=0; i1<LA; i1++)
  a1[i0][i1]=sqrule[a2[i0][i1]]-'0';

phrame(3); /* phase to show results */
}
    
/* display a screen of evolution */
evolveli(a1) int  a1[LA][LA]; {int i0, i1;
  for (i0=0; i0<LA; i0++)
  for (i1=0; i1<LA; i1++)
    videodot(i0,i1,a1[i0][i1]);
    }

// /* display a phrame around the cell array */
// phrame(i) int i; {int j;
//  for (j=0; j<LA; j++) videodot(j,PLC,i);
//  for (j=0; j<LA; j++) videodot(PLR+j,PLC+LA-1,i);
//  for (j=0; j<LA; j++) videodot(PLR,PLC+j,i);
//  for (j=0; j<LA; j++) videodot(PLR+LA-1,PLC+j,i);
// }

///* clear the cell array */
//zarry(a1) int  a1[LA][LA]; {int i0, i1;
//for (i0=0; i0<LA-1; i0++)
//for (i1=0; i1<LA-1; i1++)
//  a1[i0][i1]=0;
//}

///* generate random cells in a1 */
//ransqarr(a1) int  a1[LA][LA]; {int i0, i1, i, c;
//i=0;
//for (i0=0; i0<LA; i0++)
//for (i1=0; i1<LA; i1++)
//  a1[i0][i1]=random()%KK;
//}

/* generate random cells in the center of a1 */
rahncen(a1) int  a1[LA][LA]; {int i0, i1, ia, ib, i, c;
i=0;
for (i0=0; i0<LA; i0++)
for (i1=0; i1<LA; i1++)
  a1[i0][i1]=0;
ia=LA/3; ib=2*(LA/3);
for (i0=ia; i0<ib; i0++)
for (i1=ia; i1<ib; i1++)
  a1[i0][i1]=random()%KK;
}

/* end life.c */
