
/* SQAR21.C					*/
/* Two-Dimensional Square Lattice Life		*/
/* binary cells, first neighbors		*/
/* von Neumann neighborhoods			*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/
/* 20 July 1988 - von Neumann neighborhgood	*/

# define COLGRAF     4  /* graph resolution			*/
# define T80X25      3  /* text  resolution			*/
# define WHCYMAG     1  /* color quad for normal screen		*/
# define YELREGR     2  /* color quad for alternative screen	*/
# define AL	    80  /* edge of square of evolution		*/
# define KK	     2  /* number of states per cell		*/
# define BD	     5	/* number of neighbors incl cell	*/
# define GK         32	/* KK^BD= total number of neighborhoods	*/
# define GH         16	/* GK/KK= half number of neighborhoods	*/
# define TS	    12  /* number of symmetry classes		*/
# define NF	    50	/* lines in live frequency graph	*/
# define TRR	     2  /* row coordinate for totalistic rule	*/
# define TRC	     0	/* column coordinate for t'listic  rule	*/		
# define PLR	    60  /* row for planar array			*/
# define PLC	     0  /* column for planar array		*/

char    trule[TS];	/* evolutionary rule */
char    arule[TS];	/* auxiliary rule flags */
int     li, lj;		/* cursor in cell array */
int	lf, ln;		/* live cells, line number */
int     rs[TS];		/* rule use statistics */
int     gc[GK];		/* Golay code key array */
double  bp[BD+1];	/* Bernstein coefficients */

main() {
char c;
int  i, j;
int  arr1[AL][AL];

    videomode(COLGRAF);
    videopalette(YELREGR);

    ln=0; lf=0;
    li=AL/2; lj=AL/2;

    for (i=0; i<TS; i++) rs[i]=0;
    for (i=0; i<TS; i++) arule[i]=' ';

    videocursor(0,18,0);
    golta();
    for (i=0; i<GH; i++) gc[i+GH]=gc[i]+TS/KK;

    conlif();

    hscrrul();
    wscrrul();
    rmenu();
    frame(3);
    rancen(arr1);
    screen(arr1);
    frame(3);

    while (0<1) {					/* set up one run */
    rmenu();
    hscrrul();
    wscrrul();
    videocursor(0,0,0);
    videoputc('?',1);
    c=kbdin();
    videocursor(0,0,0);
    videoputc(c,1);
    if (c=='q') break;
    switch (c) {
	case 'g':					/* one cycle of evolution */
	    frame (1);
	    onegen(arr1);
	    screen(arr1);
	    lstat(ln%NF,lf);
	    ln++;
	    break;
        case ':':				/* multiple cycles of evolution */
	    videoscroll(12,26,24,39,0,0);
	    bpanel();
	    rstat();
	    frame (1);
	    i=0; ln=0;
            do {
	      i++;
	      videocursor(0,16,0);
	      printf("%5d",i);
	      onegen(arr1);
	      screen(arr1);
	      rstat();
	      if (ln==0) videoscroll(4,26,11,39,0,0);
	      lstat(ln,lf);
	      ln++; if (ln==NF) ln=0;
	      } while (!kbdst()); kbdin();
	    i=(int)(100.0*(((double)(lf))/((double)(AL*AL))));
	    bdot(i,i,2);
            break;
	case 's':
	    rstat();
	    break;
	case 'o':					/* translate keyboard ch */
	    printf("%o",kbdin());
	    break;
	case 't':					/* probability plot */
	    bpanel();
	    break;
        case 'r':					/* edit rule */	
	    edrule();
	    videocursor(0,4,0);
	    break;
	case 'l':					/* edit cell array */
	    edline(arr1);
	    break;
/*        case '#':					/* read stored rule */
/*	    xmenu(NX);
/*	    xtoasc(lim(1,numin(0),NX)-1);
/*	    rmenu();
/*            break;
/*	case 'D':					/* run through samples */
/*	    for (i=0; i<NX; i++) {
/*	      xmenu(i);
/*	      xtoasc(i);
/*	      ranlin();
/*	      screen(arr1);
/*	      };
/*	    videomode(T80X25);
/*	    rmenu();
/*	    break;
/*	case 'd':					/* de Bruijn diagram */
/*	    edijn();
/*	    rmenu();
/*	    lmenu();
/*	    break; */
       case 'u':				/* sparse init arry */
	    zarry(arr1);
	    arr1[AL/4][AL/4]=1;
	    arr1[AL/2+1][AL/2]=1;
	    arr1[AL/2-1][AL/2]=1;
	    arr1[AL/2][AL/2+1]=1;
	    arr1[AL/2][AL/2-1]=1;
	    screen(arr1);
	    break;
	case 'v':					/* Conway's Life */
	    conlif();
	    break;
	case 'X':					/* random rule */
	    ranrul();
	    break;
	case 'x':					/* random rule */
	    for (i=0, j=0; j<TS; j++) {
	      if (i==0) i=rand();
	      if (arule[j]==' ') {trule[j]='0'+i%KK; i/=KK;};
	      };
	    break;
	case 'y':					/* random line */
	    rancen(arr1);
	    screen(arr1);
	    break;
	case 'Y':					/* random line */
	    ranarr(arr1);
	    screen(arr1);
	    break;
	case 'z':
	    videomode(COLGRAF);
	    videopalette(YELREGR);
	    break;
/*	case 'U':					/* push rule */
/*	    if (rulptr<10) rulptr++;
/*	    for (i=0; i<KK; i++) {
/*	    rulstk[rulptr][i]=ascrule[i];
/*	    };
/*	    xmenu(rulptr);
/*	    break;
/*	case 'V':					/* pop rule */
/*	    for (i=0; i<KK; i++) {
/*	    ascrule[i]=rulstk[rulptr][i];
/*	    };
/*	    if (rulptr>0) rulptr--;
/*	    rmenu();
/*	    break;
/*	case 'G':					/* fetch rule */
/*	    for (i=0; i<KK; i++) {
/*	    ascrule[i]=rulstk[rulptr][i];
/*	    };
/*	    rmenu();
/*	    break;
/*	case '=':
/*	    for (i=1; i<8;  i++) {
/*	    for (j=0; j<SL; j++) arr1[SL*i+j]=arr1[j];};
/*	    lmenu();
/*	    break;
/*	case '~':
/*	    for (i=1; i<16;  i++) {
/*	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];};
/*	    lmenu();
/*	    break; */
        default: break;
        }; /* end switch */
    };	   /* end while */
  videomode(T80X25);
}	   /* end main */

/* calculate one generation of evolution */
onegen(arr1) int  arr1[AL][AL]; {int i0, i1, gn, arr2[AL][AL];

/* count neighbors of main square */
frame(1); /* phase to count neighbors */

for (i0=1; i0<AL-1; i0++) {
for (i1=1; i1<AL-1; i1++) {
  gn=arr1[i0-1][i1];
  gn=KK*gn+arr1[i0][i1-1];
  gn=KK*gn+arr1[i0+1][i1];
  gn=KK*gn+arr1[i0][i1+1];
  if (arr1[i0][i1]==1) gn+=GH;
  gn=gc[gn];
  arr2[i0][i1]=gn;
  rs[gn]++;
  };};

/* count neighbors for top edge (i1=0) */

for (i0=1; i0<AL-1; i0++) {
  gn=arr1[i0-1][0];
  gn=KK*gn+arr1[i0][AL-1];
  gn=KK*gn+arr1[i0+1][0];
  gn=KK*gn+arr1[i0][1];
  if (arr1[i0][0]==1) gn+=GH;
  gn=gc[gn];
  arr2[i0][0]=gn;
  rs[gn]++;
  };

/* count neighbors for bottom edge (i1=AL-1) */

for (i0=1; i0<AL-1; i0++) {
  gn=arr1[i0-1][AL-1];
  gn=KK*gn+arr1[i0][AL-2];
  gn=KK*gn+arr1[i0+1][AL-1];
  gn=KK*gn+arr1[i0][0];
  if (arr1[i0][AL-1]==1) gn+=GH;
  gn=gc[gn];
  arr2[i0][AL-1]=gn;
  rs[gn]++;
  };

/* count neighbors for left edge (i0=0) */

for (i1=1; i1<AL-1; i1++) {
  gn=arr1[AL-1][i1];
  gn=KK*gn+arr1[0][i1-1];
  gn=KK*gn+arr1[1][i1];
  gn=KK*gn+arr1[0][i1+1];
  if (arr1[0][i1]==1) gn+=GH;
  gn=gc[gn];
  arr2[0][i1]=gn;
  rs[gn]++;
  };

/* count neighbors of right edge (i0=AL-1) */

for (i1=1; i1<AL-1; i1++) {
  gn=arr1[AL-2][i1];
  gn=KK*gn+arr1[AL-1][i1-1];
  gn=KK*gn+arr1[0][i1];
  gn=KK*gn+arr1[AL-1][i1+1];
  if (arr1[AL-1][i1]==1) gn+=GH;
  gn=gc[gn];
  arr2[AL-1][i1]=gn;
  rs[gn]++;
  };

/* count neighbors for upper left hand corner */

  gn=arr1[AL-1][0];
  gn=KK*gn+arr1[0][AL-1];
  gn=KK*gn+arr1[0][1];
  gn=KK*gn+arr1[0][1];
  if (arr1[0][0]==1) gn+=GH;
  gn=gc[gn];
  arr2[0][0]=gn;
  rs[gn]++;

/* count neighbors for upper right hand corner */

  gn=arr1[AL-2][0];
  gn=KK*gn+arr1[AL-1][AL-1];
  gn=KK*gn+arr1[0][0];
  gn=KK*gn+arr1[AL-1][1];
  if (arr1[AL-1][0]==1) gn+=GH;
  gn=gc[gn];
  arr2[AL-1][0]=gn;
  rs[gn]++;

/* count neighbors for bottom left hand corner */

  gn=arr1[AL-1][AL-1];
  gn=KK*gn+arr1[0][AL-2];
  gn=KK*gn+arr1[AL-1][AL-2];
  gn=KK*gn+arr1[0][0];
  if (arr1[0][AL-1]==1) gn+=GH;
  gn=gc[gn];
  arr2[0][AL-1]=gn;
  rs[gn]++;

/* count neighbors for bottom right hand corner */

  gn=arr1[AL-2][AL-1];
  gn=KK*gn+arr1[AL-1][AL-2];
  gn=KK*gn+arr1[0][AL-1];
  gn=KK*gn+arr1[AL-1][0];
  if (arr1[AL-1][AL-1]==1) gn+=GH;
  gn=gc[gn];
  arr2[AL-1][AL-1]=gn;
  rs[gn]++;

frame(2); /* phase to update cells */
lf=0;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  arr1[i0][i1]=trule[arr2[i0][i1]]-'0';
  if (arr1[i0][i1]==1) lf++;
};};
frame(3); /* phase to show results */
}	  /* end onegen */
    
/* display a screen of evolution */
screen(arr1) int  arr1[AL][AL]; {int i0, i1;
  for (i1=0; i1<AL; i1++) {
  for (i0=0; i0<AL; i0++) {
    adot(i0,i1,arr1[i0][i1]);
    };};
}

/* display a frame around the cell array */
frame(l) int l; {int j;
for (j=0; j<AL; j++) {
  adot(0,j,l);
  adot(AL-1,j,l);
  adot(j,0,l);
  adot(j,AL-1,l);
  };
}

/* make a frame for the fixpoint graph  */
/* n     = dimension of frame           */
gfram(n) int n; {int i;
for (i=0; i<=n; i++) {bdot(i,0,0);bdot(i,n,0);bdot(n,i,0);bdot(0,i,0);};
for (i=0; i<=n; i+=10) {bdot(i,0,3);bdot(i,n,3);bdot(n,i,3);bdot(0,i,3);};
}

/* put a diagonal in the fixpoint graph */
gdiag(n) int n; {int i; for (i=0; 2*i<n; i++) bdot(2*i,2*i,3);}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
rstat() {int i, j, m, n; double s, t;
for (i=0, s=0.0; i<TS; i++) s+=(double)rs[i];
if  (s<1.0) {for (i=0; i<TS; i++) rs[i]=0; return;};
for (i=0; i<TS; i++) {
  t=333.3*(((double)rs[i])/s);	/* full scale = 1/10 population */
  if (t>=32000.0) t=50.0;	/* larger looks negative */
  n=(int)t;
  sdot(i,33,0);
  m=33; if (n>33) {n=33; m=34;};
  if (n>0) for (j=0; j<n; j++) sdot(i,j,1);
  for (j=n; j<m; j++) sdot(i,j,2);
  rs[i]=0;
  };
}

/* graph fraction of live cells */
/* y = column for dot */
/* n = value to be graphed */
lstat(y,n) int y, n; {int i, k;
k=(int)(100.0*(((double)(n))/((double)(AL*AL))));
if (k<=0) k=1; if (k>=100) k=99;
for (i=0; i<k; i++) ldot(i,y,1);
for (i=k; i<101; i++) ldot(i,y,2);
for (i=0; i<101; i++) ldot(i,y+1,0);
for (i=0; i<101; i+=10) ldot(i,y+1,3);
}

/* generate the Golay code table */
golta() {
int i, j, k;
int i0, i1, i2, i3;
int binq();
k=0;
for (i3=0; i3<KK; i3++) {
for (i2=0; i2<KK; i2++) {
for (i1=0; i1<KK; i1++) {
for (i0=0; i0<KK; i0++) {
i=binq(i0,i1,i2,i3);
j=binq(i1,i2,i3,i0);
if (j<i) i=j;
j=binq(i2,i3,i0,i1);
if (j<i) i=j;
j=binq(i3,i0,i1,i2);
if (j<i) i=j;
j=binq(i0,i3,i2,i1);
if (j<i) i=j;
j=binq(i3,i2,i1,i0);
if (j<i) i=j;
j=binq(i2,i1,i0,i3);
if (j<i) i=j;
j=binq(i1,i0,i3,i2);
if (j<i) i=j;
j=binq(i0,i1,i2,i3);
if (i==j) {gc[i]=k; k++;} else {gc[j]=gc[i];};
};};};};
/* printf("no. cl. = %3d",k); */
}

/* calculate binary equivalent */
int binq(i0,i1,i2,i3) int i0, i1, i2, i3; {return i0+KK*(i1+KK*(i2+KK*i3));}

/* calculate Golay class of a cell */
/* no check is made that cell isn't at array edge */
int golcl(i0,i1,arr1) int  i0, i1, arr1[AL][AL]; {int gn;
gn=arr1[i0-1][i1];
gn=KK*gn+arr1[i0][i1-1];
gn=KK*gn+arr1[i0+1][i1];
gn=KK*gn+arr1[i0][i1+1];
if (arr1[i0][i1]==1) gn+=GH;
gn=gc[gn];
return gn;
}

/* plot self-consistent probability graph */
bpanel() {bernc(); gfram(100); gdiag(100); bgraf(BD,100);}

/* generate Bernstein coefficients */
bernc() {int i;
for (i=0; i<BD+1; i++) bp[i]=0.0;
if (trule[0]=='1') bp[0]+=1.0;
if (trule[1]=='1') bp[1]+=4.0;
if (trule[2]=='1') bp[2]+=4.0;
if (trule[3]=='1') bp[2]+=2.0;
if (trule[4]=='1') bp[3]+=4.0;
if (trule[5]=='1') bp[4]+=1.0;
if (trule[6]=='1') bp[1]+=1.0;
if (trule[7]=='1') bp[2]+=4.0;
if (trule[8]=='1') bp[3]+=4.0;
if (trule[9]=='1') bp[3]+=2.0;
if (trule[10]=='1') bp[4]+=4.0;
if (trule[11]=='1') bp[5]+=1.0;
}

/* graph Bernstein polynomial */
bgraf(k,n) int k, n; {int i; double bern(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  bdot(i,(int)(en*bern(p,k)),1);
  };
}

/* evaluate the nth degree Bernstein polynomial at point p */
double bern(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* adot(i,j,l) - place dot on automaton screen at (i,j) in color l */
adot(i,j,l) int i, j, l; {videodot(199-PLR-(AL-1-j),PLC+60+i,l);}

/* bdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
bdot(i,j,l) int i, j, l; {videodot(199-j,219+i,l);}

/* ldot(i,j,l) - place dot on live probability  screen at (i,j) in color l */
ldot(i,j,l) int i, j, l; {videodot(90-NF+j,219+i,l);}

/* sdot(i,j,l) - place dot on statistics screen at (i,j) in color l */
sdot(i,j,l) int i, j, l; {videodot(199-10-j,70+5*i,l);}

/* clear the cell array */
zarry(arr1) int  arr1[AL][AL]; {int i0, i1;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  arr1[i0][i1]=0;
  };};
}

/* display rule menu */
rmenu() {
    scrrul(TRR,TRC);
    }

/* display rule number by sum in color */
scrrul(y,x) int x, y; {int i, n;
n=TS;
videocursor(0,y,x);
  scrstr("  ");
  for (i=0;   i<n;   i++) resch(i);
videocursor(0,y,x);
}

/* display rule number as a hexadecimal */
hscrrul() {int i, k;
  videocursor(0,0,0);
  printf("    ");
  for (i=0,  k=0; i<12;  i++) k=KK*k+trule[i]-'0';
  printf("%3.3X",k);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of cross section */
wscrrul() {int i;
  i=trule[11]-'0';
  i=KK*i+trule[10]-'0';
  i=KK*i+trule[3]-'0';
  i=KK*i+trule[1] -'0';
  i=KK*i+trule[10]-'0';
  i=KK*i+trule[9]-'0';
  i=KK*i+trule[1] -'0';
  i=KK*i+trule[0] -'0';
  videocursor(0,0,32);
  printf("     ");
  videocursor(0,0,32);
  printf("[%2d]",i);
  videocursor(0,0,0);
  }

/* edit the rule */
edrule() {char c; int i;
  i=0;
  while (0<1) {
    videocursor(0,TRR,TRC+i+2);
    videoputc(trule[i],3);
    videocursor(0,TRR,TRC+i+2);
    c=kbdin();
      if (c=='\015') break;				/* carriage return */
      switch (c) {
        case '0': case '1': 				/* state = 0,1 */
          trule[i]=c;
	  resch(i);
          i++; if (i==TS) i=0;
	  break;
        case ' ': case '\315':				/* space = advance */
	  resch(i);
          i++; if (i==TS) i=0;
	  break;
        case '\010': case '\313':			/* backspace */
	  resch(i);
          if (i>0) i--;
	  break;
	case '<':					/* cursor to front */
	  resch(i);
	  i=0;
	  break;
	case '>':					/* cursor to end */
	  resch(i);
	  i=TS-1;
	  break;
	case '\013': case '\310':			/* up arrow */
	  arule[i]='+';
	  resch(i);
          i++; if (i==TS) i=0;
	  break;
	case '\012': case '\320':			/* down arrow */
	  arule[i]=' ';
	  resch(i);
          i++; if (i==TS) i=0;
	  break;
	default:
	  break;
          };
    };
}

/* restore character in edrule */
resch(i) int i; {videoputc(trule[i],(arule[i]==' ')?1:2);}

/* edit the plane of cells */
edline(arr1) int arr1[AL][AL]; {
char c;
int  i, j, k, arr2[AL][AL], golcl();

for (i=0; i<AL; i++) for (j=0; j<AL; j++) arr2[i][j]=0;

videoscroll(19,0,24,25,0,0);
videocursor(0,19,0);
scrstr("insert states with 0,1,.");
videocursor(0,20,0);
scrstr("move cursor using arrows");
videocursor(0,21,0);
scrstr("NSEW R CGO fgmxz *<>^_");
videocursor(0,22,0);
scrstr("carriage return exits");

while (0<1) {
  adot(li,lj,3);
  videocursor(0,0,1);
  videoputc('?',1);
  c=kbdin();
  videocursor(0,0,1);
  videoputc(c,1);
  if (c == '\015') break;
  switch (c) {
    case '0': case '1':	case '.':	/* enter state */
      if (c=='.') c='1';
      i=c-'0';
      arr1[li][lj]=i;
      adot(li,lj,i);
      break;
    case '\012': case '\320':		/* down    */
      adot(li,lj,arr1[li][lj]);
      lj++; if (lj==AL) lj=0;
      break;
    case '\013': case '\310':		/* up      */
      adot(li,lj,arr1[li][lj]);
      if (lj==0) lj=AL; lj--;
      break;
    case '\014': case '\315': case ' ':	/* forward */
      adot(li,lj,arr1[li][lj]);
      li++; if (li==AL) li=0;
      break;
    case '\010': case '\313':		/* back    */
      adot(li,lj,arr1[li][lj]);
      if (li==0) li=AL; li--;
      break;
    case '<':				/* left margin  */
      adot(li,lj,arr1[li][lj]);
      li=0;
      break;
    case '>':				/* right margin */
      adot(li,lj,arr1[li][lj]);
      li=AL-1;
      break;
    case '^':				/* top margin  */
      adot(li,lj,arr1[li][lj]);
      lj=0;
      break;
   case '_':				/* bottom margin  */
      adot(li,lj,arr1[li][lj]);
      lj=AL-1;
      break;
   case '*':				/* bottom margin  */
      adot(li,lj,arr1[li][lj]);
      li=AL/2; 
      lj=AL/2;
      break;
     case 'f':				/* generate a frame */
      frame (2);
      break;
    case 'g':				/* one cycle of evolution */
      frame (1);
      onegen(arr1);
      screen(arr1);
      break;
    case 'm':				/* mark neighborhood */
      arule[golcl(li,lj,arr1)]='+';
      scrrul(TRR,TRC);
      break;
    case 'M':				/* mark neighborhood over a region */
      if (li<10||li>AL-11) break;
      if (lj<10||lj>AL-11) break;
      for (i=li-10; i<li+10; i++) {
      for (j=lj-10; j<lj+10; j++) {
        arule[golcl(i,j,arr1)]='+';
        };};
      scrrul(TRR,TRC);
      break;
    case 'x':				/* uncomitted transitions */
      for (k=0; k<TS; k++) arule[k]=' ';
      videocursor(0,TRR,TRC);
      scrrul(TRR,TRC);
      break;
    case 'z':				/* clear array  */
      zarry(arr1);
      screen(arr1);
      break;
    case 'N':				/* rotate array up */
      for (i=0; i<AL; i++) {
        k=arr1[i][0]; 
        for (j=1; j<AL; j++) arr1[i][j-1]=arr1[i][j];
        arr1[i][AL-1]=k;
	};
      if (lj==0) lj=AL; lj--;
      screen(arr1);
      break;
    case 'S':				/* rotate array down */
      for (i=0; i<AL; i++) {
        k=arr1[i][AL-1];
        for (j=1; j<AL; j++) arr1[i][AL-j]=arr1[i][AL-j-1];
        arr1[i][0]=k;
	};
      lj++; if (lj==AL) lj=0;
      screen(arr1);
      break;
    case 'E':				/* rotate array right */
      for (j=0; j<AL; j++) {
        k=arr1[AL-1][j];
        for (i=1; i<AL; i++) arr1[AL-i][j]=arr1[AL-i-1][j];
        arr1[0][j]=k;
	};
      li++; if (li==AL) li=0;
      screen(arr1);
      break;
    case 'W':				/* rotate array left */
      for (j=0; j<AL; j++) {
        k=arr1[0][j];
        for (i=1; i<AL; i++) arr1[i-1][j]=arr1[i][j];
	arr1[AL-1][j]=k;
	};
      if (li==0) li=AL; li--;
      screen(arr1);
      break;
    case 'R':				/* reflect array in diagonal */
      for (j=1; j<AL; j++) {
      for (i=0; i<j;  i++) {
        k=arr1[i][j];
	arr1[i][j]=arr1[j][i];
        arr1[j][i]=k;
        };};
      k=li; li=lj; lj=k;
      screen(arr1);
      break;
    case 'C':				/* save cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr2[i][j]=arr1[i][j];
	};};
      break;
    case 'G':				/* recover saved cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr1[i][j]=arr2[i][j];
	};};
      screen(arr1);
      break;
    case 'O':				/* or saved cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr1[i][j]|=arr2[i][j];
	};};
      screen(arr1);
      break;
    default:
      break;
      };				/* end switch */
  };					/* end while */
adot(li,lj,arr1[li][lj]);
videoscroll(19,0,24,25,0,0);
videocursor(0,0,1);
videoputc(' ',1);
}				        /* end edline */

/* generate the transitions for Banks' computer */
conlif() {int i;
  for (i=0; i<TS; i++) trule[i]='0';
  trule[4]='1';
  trule[5]='1';
  trule[6]='1';
  trule[7]='1';
  trule[9]='1';
  trule[10]='1';
  trule[11]='1';
}

/* generate random cells in arr1 */
ranarr(arr1) int  arr1[AL][AL]; {int i0, i1, i, c;
i=0;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  if (i%8 == 0) c=rand(); i++;
  arr1[i0][i1]=c%KK; c/=KK;
  };};
}

/* generate random cells in the center of arr1 */
rancen(arr1) int  arr1[AL][AL]; {int i0, i1, ia, ib, i, c;
i=0;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  arr1[i0][i1]=0;
  };};
ia=AL/3; ib=2*(AL/3);
for (i0=ia; i0<ib; i0++) {
for (i1=ia; i1<ib; i1++) {
  if (i%8 == 0) c=rand(); i++;
  arr1[i0][i1]=c%KK; c/=KK;
  };};
}

/* generate a random rule */
ranrul() {int i0, i, c;
i=0;
for (i0=0; i0<TS; i0++) {
  if (i%8 == 0) c=rand(); i++;
  trule[i0]=c%KK+'0'; c/=KK;
};
}

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}

/* write a string in graphics mode */
scrstr(s) char *s;
  {for (; *s != '\0'; s++) videoputc(*s,1); }

/* keyboard status */
int kbdst() {return(bdos(11) & 0xFF);}

/* direct keyboard input, no echo */
kbdin() {int c;
  if ((c = bdos(7) & 0xFF) == '\0') c = (bdos(7) & 0xFF) | 0x80;
  return(c);
}

/* read number from keyboard */
int numin(n) int n; {char c;
  c=kbdin();
  if (c>='0'&&c<='9') return(numin(10*n+(c-'0'))); else return(n);
}

/* end */
